import numpy as np
import scipy as sp

from lib.pysit_interface import ForwardSolver
from lib.inversion_solver import InverseSolver
from util.simulationcontext import SimulationContext

import lib.inversion_solver
lib.inversion_solver.DEBUG = True


class FWDResult_DEBUG:
    def __init__(self, shot):
        self.shot = shot

def create_example_material(mesh, domain):
    def _gaussian_derivative_pulse(XX, YY, threshold, **kwargs):
        """ Derivative of a Gaussian at a specific sigma """
        T = -5 * YY * np.exp(-(YY ** 2 * 10))
        T[np.where(abs(T) < threshold)] = 0
        return T

    [XX, YY] = mesh.mesh_coords()
    material = 1 * np.ones(mesh.shape())

    material += _gaussian_derivative_pulse(XX, (YY - 30)/5, 1e-7)
    return 10 * material

source_pos = (40.13251783893986, 6.477272727272727)
receiver_pos = [(52.330275229357795, 4.534090909090909), (55.869520897043834, 4.534090909090909), (59.408766564729866, 4.534090909090909), (62.9480122324159, 4.534090909090909), (66.48725790010194, 4.534090909090909), (70.02650356778797, 4.534090909090909), (73.56574923547402, 4.534090909090909)]

# N = 10
# receiver_pos = [(52.330275229357795 + i*(73.56574923547402-52.330275229357795)/(N-1), 4.534090909090909) for i in range(N)]

FWD = ForwardSolver(
    dimensions=((0.0, 100.0), (0.0, 70.0)), grid_resolution=(100, 70), trange=(0, 10),
    )

mesh, domain = FWD.mesh(), FWD.domain()
material = create_example_material(mesh, domain)
wavefields, s_obs, shot, retval = FWD.solve(material, source_position=source_pos, receiver_positions=receiver_pos,
                                            update_shot=True)
frames = FWD.animate(wavefields, frame_count=30)

#shot.receivers.data = s_obs
#print(shot)

FWD.plotSeismogram(s_obs, (800, 300)).save("/tmp/seismogram_real.png")
FWD.plotSeismogram(shot.receivers.interpolate_data(FWD._solver.ts()), (800, 300)).save("/tmp/seismogram_real_XXXX.png")


for i, f in enumerate(frames):
    f.save("/tmp/REAL{:05d}.png".format(i))


ctx = SimulationContext()
ctx.gridWidth, ctx.gridHeight = FWD.Nx, FWD.Ny
ctx.waterHeight = 0
ctx.simulationHeight = ctx.gridHeight
ctx.fwdResult = FWDResult_DEBUG(shot)
ctx.FWDSolver = FWD
ctx.emitterPosition = source_pos
ctx.receiverPositions = receiver_pos


IWS = InverseSolver(ctx, 10*np.ones_like(material))
#IWS = InverseSolver(shot, 10*np.ones_like(material), FWD, source_pos, receiver_pos)
# IWS = InverseSolver(shot, material.copy(), FWD, source_pos, receiver_pos)

# inv_solv_thread = InverseSolverThread(IWS)
# inv_solv_thread.start()

IWS.run()

# IWS.join()
