from PyQt5.Qt import QRect
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QImage, QPainter, QPalette, QColor, QFont
from PyQt5.QtWidgets import QWidget
from lib.constants import Orientation, ResPath
from util import renderutil
from widgets.materialeditor.dragobject import DragObject

class SignalPlacement(QWidget):
    WIDGET_HEIGHT = 132
    SHIP_HEIGHT = 30
    
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.setMinimumHeight(SignalPlacement.WIDGET_HEIGHT)
        self.setMaximumHeight(SignalPlacement.WIDGET_HEIGHT)
        
        palette = QPalette()
        palette.setColor(QPalette.Background, Qt.white)
        self.setPalette(palette)

        self._bg = QImage(ResPath.textures + "water.png")

        self._emitter = DragObject(self, orientation=Orientation.HORIZONTAL)
        self._emitter.setGeometry(100, 0, 45, SignalPlacement.SHIP_HEIGHT)
        self._emitter.setImage(QImage(ResPath.textures + "boat.png"))

        self._receivers_count = 19
        self._receivers = DragObject(self, orientation=Orientation.HORIZONTAL)
        receiverImage = QImage(ResPath.textures + "receiver.png")
        self.receiversHeight = 20
        self.receiversTextHeight=30
        receiversWidth = receiverImage.width() * self.receiversHeight // receiverImage.height() * self._receivers_count
        self._receivers.setGeometry(400, SignalPlacement.SHIP_HEIGHT, receiversWidth, self.receiversHeight+self.receiversTextHeight)
        size = self._receivers.size()
        size.setHeight(self.receiversHeight)
        image = renderutil.createFilledImage(receiverImage, size , tilesX=self._receivers_count, tilesY=1)
        
        number_image = QImage(receiversWidth, self.receiversTextHeight, image.format())
        number_image.fill(0)
        numbers_rect = number_image.rect()
        
        painter = QPainter(number_image)
        painter.setPen(Qt.white)
        painter.setFont(QFont("Times", 14, QFont.Bold))
        painter.drawText(numbers_rect, Qt.AlignLeft, " 1")
        painter.drawText(numbers_rect, Qt.AlignCenter, str((self._receivers_count+1)//2))
        painter.drawText(numbers_rect, Qt.AlignRight, str(self._receivers_count))
        painter.end()
        
        image = renderutil.mergeImages([image, number_image])
        
        self._receivers.setImage(image)

    def paintEvent(self, event):
        QWidget.paintEvent(self, event)
        painter = QPainter(self)
        painter.setOpacity(0.8)
        painter.drawImage(QRect(0, SignalPlacement.SHIP_HEIGHT, self.width(), self.height() - SignalPlacement.SHIP_HEIGHT), self._bg)
        
    def waterHeight(self):
        return self.height() - SignalPlacement.SHIP_HEIGHT

    def emitterPosition(self, grid_width, water_grid_height):
        x = float(2 * self._emitter.x() + self._emitter.width()) / (2 * self.width())
        #y = float(2 * self._emitter.y() + self.receiversHeight+self.receiversTextHeight) / (2 * self.height())
        y = 0.5
        return (grid_width * x, water_grid_height * y)

    def receiversPositions(self, grid_width, water_grid_height):
        rel_x_left = float(self._receivers.x()) / self.width()
        rel_x_right = float(self._receivers.x() + self._receivers.width()) / self.width()
        rel_receiver_width = (rel_x_right - rel_x_left) / self._receivers_count
        rel_receivers_xs = [rel_x_left + rel_receiver_width * (0.5 + i) for i in xrange(self._receivers_count)]
        y = float(self.receiversHeight) / (2 * self.height()) * water_grid_height
        return [(rel_x * grid_width, y) for rel_x in rel_receivers_xs]

#     def resizeEvent(self, event):
#         print(event.isAccepted())
#         if event.oldSize().width() < 0 or event.oldSize().height() < 0:
#             return
#
#         for child in self.children():
#             if not child.xpercent:
#                 continue
#             child.move(child.xpercent * self.width(), child.pos().y())
