from PyQt5.Qt import Qt
from PyQt5.QtCore import QPoint
from PyQt5.QtGui import QPen, QColor, QPainterPath
from PyQt5.QtWidgets import QGraphicsView, QGraphicsScene, QGraphicsPathItem, QGraphicsLineItem
from scipy.interpolate import interp1d


class WaveDrawing(QGraphicsView):
    def __init__(self, parent=None):
        super(QGraphicsView, self).__init__()
        self.scene = QGraphicsScene(0, 0, 500, 500)
        self.painterpath = QPainterPath(QPoint(0, 250))
        for i in range(500):
            self.painterpath.lineTo(i, 250)

        self.pathitem = QGraphicsPathItem(self.painterpath)
        self.scene.addItem(self.pathitem)

        line = QGraphicsLineItem(0, 250, 500, 250)
        pen = QPen(QColor(150, 150, 150))
        pen.setStyle(Qt.DashLine)
        line.setPen(pen)
        self.scene.addItem(line)

        self.setScene(self.scene)
        self.setMinimumSize(500, 500)

        self._isPressed = False
        self._lastchange = 0, 0

    def changePoint(self, x, y, first=False):
        if not (0 <= x < 500 and 0 <= y < 500):
            return
        print("change", x, y)

        oldx = x if first else self._lastchange[0]
        minx = min(x, oldx)
        maxx = max(x, oldx) + 1
        for xx in range(int(minx), int(maxx)):
            self.painterpath.setElementPositionAt(xx, xx, y)
            self.scene.removeItem(self.pathitem)
            self.pathitem = QGraphicsPathItem(self.painterpath)
            self.scene.addItem(self.pathitem)
        self._lastchange = x, y

    def smoothen(self):
        print("smoothen")

        xs = list(range(0, 501, 5))
        ys = [self.painterpath.elementAt(xx).y for xx in xs]

        ys = [250, 250] + [sum(ys[i - 2:i + 3]) / 5 for i in range(2, 99)] + [250, 250]

        # print(len(xs),len(ys))

        f = interp1d(xs, ys, kind="cubic")

        painterpath = QPainterPath(QPoint(0, 250))
        for i in range(501):
            painterpath.lineTo(i, f(i))

        self.scene.removeItem(self.pathitem)
        self.painterpath = painterpath
        self.pathitem = QGraphicsPathItem(self.painterpath)
        self.scene.addItem(self.pathitem)



    def mousePressEvent(self, event):
        print("press", self._isPressed)

        if event.button() == Qt.LeftButton:
            self._isPressed = True
            pos = self.mapToScene(event.pos())
            x, y = pos.x(), pos.y()
            self.changePoint(x, y, first=True)

        if event.button() == Qt.RightButton:
            for i in range(3):
                self.smoothen()

    def mouseReleaseEvent(self, event):
        self._isPressed = False

    def mouseMoveEvent(self, event):
        if self._isPressed:
            pos = self.mapToScene(event.pos())
            x, y = pos.x(), pos.y()
            self.changePoint(x, y)

