from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QApplication, QMainWindow, QPushButton
from util import utils
import sys
import time
from multiprocessing.pool import Pool
import numpy as np



"""
http://doc.qt.io/qt-4.8/qt-threads-mandelbrot-example.html
https://stackoverflow.com/questions/15675043/multiprocessing-and-gui-updating-qprocess-or-multiprocessing
"""
class WorkerThread(QThread):
    finished = pyqtSignal(object)

    def __init__(self, parent, priority=QThread.LowPriority):
        QThread.__init__(self, parent)
        self._func = None
        self._args = None
        self._kwargs = None
        self._priority = priority

    def setPriority(self, priority):
        self._priority = priority

    def execute(self, func, *args, **kwargs):
        if self.isRunning():
            print "cant execute '"+func.__name__+"' on", self, ", already executing'"+self._func.__name__+"'"
            return
        print "now executing '"+func.__name__+"' on", self
        self._func = func
        self._args = args
        self._kwargs = kwargs
        self.start(self._priority)

    def run(self):
        result = self._func(*self._args, **self._kwargs)
        self.finished.emit(result)

#     def __del__(self):
#         print "test"
#         self.terminate()
        

# example
class Window(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.setFixedSize(400, 200)
        QPushButton("Start direct", parent=self, clicked=self.startDirect)
        b1Worker = QPushButton("Start 1 worker", parent=self, clicked=self.start1Worker)
        b1Worker.move(0, 40)
        b2Worker = QPushButton("Start 2 workers", parent=self, clicked=self.start2Workers)
        b2Worker.move(0, 80)
        
        bMultiP = QPushButton("Start multiprocessing", parent=self, clicked=self.startMultiP)
        bMultiP.setGeometry(120, 0, 200, 30)
        bMultiPAsync = QPushButton("Start multiprocessing async", parent=self, clicked=self.startMultiPAsync)
        bMultiPAsync.setGeometry(120, 40, 200, 30)
        
        self._worker = WorkerThread(self)
        self._worker.finished.connect(self.accum)
        self._worker2 = WorkerThread(self)
        self._worker2.finished.connect(self.accum)

        self.startTime = 0
        self.accumulator = 0

    def startDirect(self):
        print "\nstarting direct"
        self.startTime = time.clock()
        # both computed by self
        solve()
        solve2()
        print "direct took {0:.3f} ms".format((time.clock() - self.startTime) * 1000.0)

    def start1Worker(self):
        print "\nstarting with 1 worker"
        self.startTime = time.clock()
        # 1 for the worker
        self._worker.execute(solve)
        # 2 for self
        solve2()
        self.accum(None)

    def start2Workers(self):
        print "\nstarting with 2 workers"
        self.startTime = time.clock()
        # 1 for worker1
        self._worker.execute(solve)
        # 2 for worker2
        self._worker2.execute(solve2)

    def startMultiP(self):
        print "\nstarting multiprocessing"
        pool = Pool(processes=2)
        self.startTime = time.clock()
        pool.apply(solve)
        pool.apply(solve2)
        pool.close()
        pool.join()
        print "multiprocessing took {0:.3f} ms".format((time.clock() - self.startTime) * 1000.0)

    def startMultiPAsync(self):
        print "\nstarting multiprocessing async"
        pool = Pool(processes=2)
        self.startTime = time.clock()
        pool.apply_async(solve)
        pool.apply_async(solve2)
        pool.close()
        pool.join()
        print "multiprocessing async took {0:.3f} ms".format((time.clock() - self.startTime) * 1000.0)
        
    def accum(self, result):
        print "result arrived"
        self.accumulator += 1
        if self.accumulator >= 2:
            print "worker(s) took {0:.3f} ms".format((time.clock() - self.startTime) * 1000.0)
            self.accumulator = 0

def solve():
    print "solve"
    a = np.ones(shape=(1000, 1000), dtype=np.float32)
    for i in range(10000):
        a *= a
    return None

def solve2():
    print "solve2"
    a = np.ones(shape=(1000, 1000), dtype=np.float32)
    for i in range(5000):
        a *= a
    return None

if __name__ == "__main__":
    # to show exceptions from Qt after crash
    sys.excepthook = utils.excepthook

    # create the QApplication
    app = QApplication(sys.argv)
    window = Window()
    window.show()
    sys.exit(app.exec_())
