from PyQt5.QtCore import QLineF, Qt, QRectF
from PyQt5.QtGui import QColor, QPainter
from PyQt5.QtWidgets import QWidget

class SeismogramWidget(QWidget):
    HEIGHT = 200
    
    def __init__(self, slider, isForward = False,parent=None):
        super(QWidget, self).__init__(parent)
        self.setFixedHeight(SeismogramWidget.HEIGHT)
        self._optimal_seismogram = None
        self._current_seismogram = None
        self.slider = slider
        
        self._isForward = isForward
        
    def set_original_seismogram(self, seismogram):
        self._original_seismogram = seismogram
        self.update()   
        
    def set_current_seismogram(self,seismogram):
        self._current_seismogram = seismogram
        self.update()    
        
    def mousePressEvent(self, ev):
        self.slider.mousePressEvent(ev)

    def mouseMoveEvent(self, ev):
        self.slider.mouseMoveEvent(ev)
    
    def set_isForward(self, isForward):
        self._isForward = isForward
        
    def paintEvent(self, event):
        QWidget.paintEvent(self, event)

        painter = QPainter(self)
        
        painter.setPen(QColor(Qt.lightGray))
        x = self.slider.getHandleX()
        painter.drawLine(QLineF(x, 0, x, self.height()))
            
        if self._original_seismogram:
            painter.drawImage(self.rect(), self._original_seismogram)
    
        if self._current_seismogram is not None:            
            if self._isForward:
                tarRect = QRectF(self.rect())
                #perc = self.slider.value() / float(self.slider.maximum())
                perc = x * 1.0/ self.slider.width()
                tarRect.setWidth(tarRect.width() * perc)
                srcRect = QRectF(self._current_seismogram.rect())
                srcRect.setWidth(srcRect.width() * perc)
                painter.drawImage(tarRect, self._current_seismogram, srcRect)
            else:
                painter.drawImage(self.rect(), self._current_seismogram)


            
            
                      
            
